﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2018 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.ComponentModel;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment {
  [Item("GQAPAssignmentArchive", "Stores the pareto front of assignments.")]
  [StorableClass]
  public sealed class GQAPAssignmentArchive : Item, INotifyPropertyChanged {

    [Storable]
    private ItemList<GQAPSolution> solutions;
    public ItemList<GQAPSolution> Solutions {
      get { return solutions; }
      set {
        if (solutions == value) return;
        solutions = value;
        OnPropertyChanged(nameof(Solutions));
      }
    }

    [Storable]
    private GQAPInstance problemInstance;
    public GQAPInstance ProblemInstance {
      get { return problemInstance; }
      set {
        if (problemInstance == value) return;
        problemInstance = value;
        OnPropertyChanged(nameof(ProblemInstance));
      }
    }

    [StorableConstructor]
    private GQAPAssignmentArchive(bool deserializing) : base(deserializing) { }
    private GQAPAssignmentArchive(GQAPAssignmentArchive original, Cloner cloner)
      : base(original, cloner) {
      solutions = cloner.Clone(original.solutions);
      problemInstance = cloner.Clone(original.problemInstance);
    }
    public GQAPAssignmentArchive()
      : base() {
      this.solutions = new ItemList<GQAPSolution>();
    }
    public GQAPAssignmentArchive(GQAPInstance problemInstance)
      : this() {
      this.problemInstance = problemInstance;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new GQAPAssignmentArchive(this, cloner);
    }

    public event PropertyChangedEventHandler PropertyChanged;
    private void OnPropertyChanged(string propertyName) {
      var handler = PropertyChanged;
      if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
    }
  }
}
