﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2018 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment {
  [Item("N-Move Absolute TabuAttribute", "Attribute to determine whether a certain move is tabu.")]
  [StorableClass]
  public class NMoveAbsoluteTabuAttribute : Item {

    [Storable]
    public List<Tuple<int, int>> OldAssignments { get; private set; }
    [Storable]
    public double MoveQuality { get; private set; }

    [StorableConstructor]
    protected NMoveAbsoluteTabuAttribute(bool deserializing) : base(deserializing) { }
    protected NMoveAbsoluteTabuAttribute(NMoveAbsoluteTabuAttribute original, Cloner cloner) : base(original, cloner) { }
    public NMoveAbsoluteTabuAttribute(NMove move, IntegerVector assignment, double moveQuality)
      : base() {
      OldAssignments = new List<Tuple<int, int>>(move.N);
      foreach (var equip in move.Indices)
        OldAssignments.Add(Tuple.Create(equip, assignment[equip]));
      MoveQuality = moveQuality;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new NMoveAbsoluteTabuAttribute(this, cloner);
    }
  }
}
